extern crate chrono;
extern crate colored;
extern crate dirs;
#[macro_use]
extern crate log;
extern crate serde;
#[macro_use]
extern crate serde_derive;
extern crate toml;

mod app;
mod apps;
mod logger;
mod proc;
mod result;

use std::path::PathBuf;

use dirs::config_dir;

use apps::Apps;
use proc::Proc;
use result::Result;

fn main() {
	if let Err(e) = logger::init() {
		println!("FATAL: Error initializing logger: {:?}", e);
		std::process::exit(1);
	}

	let apps_file: PathBuf = match config_dir() {
		Some(mut d) => {
			d.push("sup.toml");
			d
		}
		None => PathBuf::from("sup.toml"),
	};

	match startup(apps_file) {
		Ok(p) => {
			info!("shutting down");
			for mut proc in p {
				proc.stop();
			}
		}
		Err(e) => {
			error!("FATAL: {:?}", e);
			std::process::exit(1);
		}
	};
}

fn startup(apps_file: PathBuf) -> Result<Vec<Proc>> {
	let apps = Apps::load(apps_file)?;
	let (procs, holds) = apps.start();
	let procs = apps.run(procs, holds)?;
	Ok(procs)
}
